# ./addons/eventscripts/freezetag/freezetag.py

import es
import gamethread
import playerlib
import time
import popuplib
from usermsg import hudhint
from popuplib import create


ezrestrict = es.import_addon('ezrestrict')

###

SECONDS_TO_UNFREEZE = 3
UNFREEZE_CHECK = 0.5 # In seconds
UNFREEZE_DISTANCE = 150 # In game units
# remove_scout = 0 # Deletes the Scout if set to 1

explanasion = {1:'''Ein getroffener Spieler friert ein,
                    er kann sich nicht mehr bewegen und auch nicht schiessen.
                    Um diesen eingefrorenen Spieler dann wieder aufzutauen,
                    muss ein Team-Mate fuer mindestens drei Sekunden in seiner Naehe stehen.
                    Push-Schuss,
                    spring und schiesse in den Boden um einen Push zu erhalten''',
               2:'''A hit freezes the enemy,
                    he can\'t move or shoot anymore.
                    To defrost that player again a Team-Mate has to stand
                    near him for 3 seconds
                    Push-Shoot,
                    jump and shoot into the ground to get an upwards boost'''}
                    
crosshairlist = {"None":0,
                 "Dot":"Crosshair\\crosshair3.vmt",
                 "Cross":"Crosshair\\crosshair4.vmt",
                }
###


WeaponList = {}
PlayerCoords = {}
playerchoices = {}


class Unfreeze(dict):
    def __init__(self):
        super(Unfreeze, self).__init__()
        self.start()

    def start(self):
        self.cancel()
        self.clear()
        self.step()

    def cancel(self):

        gamethread.cancelDelayed('freezetag')

    def step(self):
        end = 0
        ts = playerlib.getPlayerList('#t')
        cts = playerlib.getPlayerList('#ct')
       
        tc = 0
        for x in ts:
            if x.getFreeze():
                tc += 1
        if int(len(ts)) == tc:
            end = 1
            for x in ts:
                x.slay()
       
        if end == 0:
            ctc = 0
            for x in cts:
                if x.getFreeze():
                    ctc += 1
                    if int(len(cts)) == ctc:
                        end = 1
                        for x in cts:
                            x.slay()

        playerlist = playerlib.getPlayerList('#alive')
        
        if not end:
            for player in playerlist:
                if not player.userid in self and not player.freeze:
                    continue
                beingunfrozen = False
                for subplayer in playerlist:
                # Cannot unfreeze yourself or be unfrozen by multiple players
                    if player.userid in self and self[player.userid][0] != subplayer.userid:
                        continue

                    if player is not subplayer and player.team == subplayer.team and not subplayer.freeze:
                        if player.getDistance(subplayer) <= UNFREEZE_DISTANCE:
                            beingunfrozen = True
                            if player.userid not in self:
                                self[player.userid] = (subplayer.userid, time.time())
                                # Play Freeze Sound
                                es.cexec_all('play freezetag/entfreeze.wav')

                            elapsed = time.time() - self[player.userid][1]
                            
                            if elapsed < SECONDS_TO_UNFREEZE:
                                # Update HUD
                                elapsed_percent = min(1, elapsed / SECONDS_TO_UNFREEZE * 100)
                                progress_bar    = '[]' * int(elapsed_percent // 10)
                                if elapsed_percent % 10 >= 5:
                                    progress_bar += '['

                                line1 = 'Unfreezing ' + player.name + '...'
                                hudhint(subplayer, line1.center(len(progress_bar)) + '\n' + progress_bar.center(len(line1)))
                                line1 = 'Being unfrozen by ' + subplayer.name + '...'
                                hudhint(player, line1.center(len(progress_bar)) + '\n' + progress_bar.center(len(line1)))

                            else:
                                # Give UnFreezer a point
                                es.server.queuecmd('score add %s 1'%subplayer.userid)
                                es.tell(subplayer.userid,"#multi","#greenYou unfroze #lightgreen%s" % player.name)
                                es.tell(player.userid,"#multi","#greenYou were unfrozen by #lightgreen%s" % subplayer.name)
                                # Unfreeze
                                del self[player.userid]
                                player.freeze = 0
                        
                                # Play a Sound at Unfreeze
                                es.cexec_all('play freezetag/freeze.wav')
                                player.color = (255, 255, 255)
                                player.godmode(0)
                                # player.slay()
                                es.server.queuecmd("est_spawn %s 1" % player.userid)
                                break
                                
                # reset their timer if no one is unfreezing them
                if not beingunfrozen:
                    if player.userid in self:
                        del self[player.userid]

        gamethread.delayedname(UNFREEZE_CHECK, 'freezetag', self.step)



unfreeze = Unfreeze()



###

class Info(object):
    def __init__(self):
        self.infoLang = create('info_languagePopup')
        self.infoBox  = create('info_boxPopup')
        self.infoLang.addline('Info - Choose Language')
        self.infoLang.addline('---------------')
        self.infoLang.addline(' ')
        self.infoLang.addline('->1. German')
        self.infoLang.addline('->2. English')
        self.infoLang.addline('---------------')
        self.infoLang.addline(' ')
        self.infoLang.addline('0. Close')
        self.infoLang.menuselect = self.langMenuSelect
        self.infoBox.addline('Freezetag')
        self.infoBox.addline('---------------')
        self.infoBox.addline(' ')
        self.infoBox.addline('')
        self.infoBox.addline('---------------')
        self.infoBox.addline(' ')
        self.infoBox.addline('0. Close')
        es.regsaycmd('/info', 'info/infopopup')
    def scriptUnload(self):
        self.infoLang.delete()
        self.infoBox.delete()
        es.unregsaycmd('/info')
        gamethread.cancelDelayed("advertloopinfo")
    def langMenuSelect(self, userid, choice, popupid):
        if choice in (1, 2):
            self.infoBox.modline(4, explanasion[choice])
            self.infoBox.send(userid)
    def timeCheck(self):
        gamethread.cancelDelayed("advertloopinfo")
        gamethread.delayedname(60,"advertloopinfo",self.timeCheck)
        es.msg('#multi', "#green[#lightgreenInfo#green] #defaultType '/info' to open up an explanation-box.")
            
info = Info()

def infopopup():
    info.infoLang.send(es.getcmduserid())
    
###



def load():
    ezrestrict.restrict('@all','#all')
    ezrestrict.unrestrict('@all',['scout','knife'])
    # If this addon is loaded while players are connected
    # we want to set each player's health
    for player in playerlib.getPlayerList('#alive'):
        player.setHealth(1124)

    es.stringtable('downloadables', 'sound/freezetag/freeze.wav')
    es.stringtable('downloadables', 'sound/freezetag/entfreeze.wav')
    
    gamethread.delayedname(2,"crosshairloop",loop)
    es_map_start(0)


def unload():
    info.scriptUnload()
    gamethread.cancelDelayed("crosshairloop")
    
def es_map_start(ev):
    for x in crosshairlist:
        if crosshairlist[x] == 0:
            continue
        es.stringtable('downloadables', 'materials\\' + crosshairlist[x])
        es.stringtable('downloadables', 'materials\\' + crosshairlist[x][:-2] + "tf")
    es.stringtable('downloadables', 'sound/freezetag/freeze.wav')
    es.stringtable('downloadables', 'sound/freezetag/entfreeze.wav')
    
def round_start(event_var):
    global roundended
    roundended = 0
    # Restart the timer
    unfreeze.start()
    es.msg("#multi","#greenType #default!crosshair #greento choose your crosshairstyle")

def player_spawn(event_var):
    if not int(event_var['es_userteam']): return
    uid = int(event_var["userid"])
    myPlayer = playerlib.getPlayer(uid)
    myPlayer.setHealth(1124)
    # if they respawn as frozen
    if uid in PlayerCoords and not roundended:
        co = PlayerCoords[uid]
        # strip their weapons
        es.server.queuecmd("es_setpos %s %s %s %s" % (uid,co[0],co[1],co[2]))
        es.server.queuecmd("est_stripplayer %s" % uid)
        es.server.queuecmd("est_removeidle weapon_scout")
        del PlayerCoords[uid]
        
        # Freeze the player
        myPlayer.godmode(1)
        gamethread.delayedname(.3, 'checkFreeze_%s'%myPlayer.userid, checkFreeze, myPlayer)
        
        # put them into third person
        es.setplayerprop(uid,"CCSPlayer.baseclass.m_iObserverMode",1)
        es.setplayerprop(uid,'CBasePlayer.m_hObserverTarget',0)
        
        # Color the player according to his team
        myTeam = int(event_var['es_userteam'])
        if myTeam == 3:
            myPlayer.setColor(0, 0, 255)
       
        elif myTeam == 2:
            myPlayer.setColor(255, 0, 0)

        # If all players on the team are frozen the other team wins!
        
        for player in playerlib.getPlayerList('#alive'):
            if player.team == myTeam and not player.freeze:
                return

        # All players on the team are frozen so slay them
        round_end(0)
        for player in playerlib.getPlayerList('#alive'):
            if player.team == myTeam:
                # We need to change their color back as well
                player.color = (255, 255, 255)
                player.slay()
    else:
        es.server.queuecmd('es_xgive %s weapon_scout'%event_var['userid'])
        es.server.queuecmd('es_delayed 0.2 es_sexec %s "use weapon_scout"'%event_var['userid'])
        # es.cexec(event_var['userid'],"use weapon_scout")
        # We set each player's health so the players don't die
    
        es.setplayerprop(event_var['userid'],"CCSPlayer.baseclass.m_iObserverMode",0)

def player_hurt(event_var):
    # We don't want players to die
    myPlayer = playerlib.getPlayer(event_var['userid'])
    myPlayer.setHealth(1124)
    # If this is world damage and the player is frozen the player needs to
    # die to keep from getting stuck
    if int(event_var["dmg_health"]) > 7124:
        return
        
    attacker = int(event_var['attacker'])
    if not attacker and myPlayer.freeze:
        # and change their color back
        myPlayer.color = (255, 255, 255)
        myPlayer.slay()
        return

   # If world damage, don't freeze the player
    if not attacker:
        return
    # If the player is not already frozen we need to freeze him or her
    
    if event_var["es_userteam"] == event_var["es_attackerteam"]:
        return

    if myPlayer != attacker and not myPlayer.freeze:
        PlayerCoords[int(myPlayer)] = myPlayer.getLocation()
        # Give the attacker a point
        es.server.queuecmd('damage %s 9999 1 %s'%(myPlayer.userid,attacker))
        es.server.queuecmd('es_delayed 0.1 est_deleteragdolls %s'% myPlayer.userid)

        # Tell each player what happened
        es.tell(attacker, '#multi', '#defaultYou froze#lightgreen ' + event_var['es_username'] + '#default!')
        es.tell(myPlayer, '#multi', '#defaultYou were frozen by#lightgreen ' + event_var['es_attackername'] + '#default!')





def checkFreeze(player):
    if player.onGround():
        player.freeze = 1
    else:
        gamethread.cancelDelayed('checkFreeze_%s'%player.userid)
        gamethread.delayedname(.2, 'checkFreeze_%s'%player.userid, checkFreeze, player)



def player_disconnect(event_var):
    userid = int(event_var['userid'])
    if userid in playerchoices:
        del playerchoices[userid]
    if userid in PlayerCoords:
        del PlayerCoords[userid]
    gamethread.cancelDelayed('checkFreeze_%s'%event_var['userid'])
    if userid in WeaponList:
        del WeaponList[userid]
    for player in list(unfreeze):
        if unfreeze[player][0] == userid:
            del unfreeze[player]
            break


roundended = 0
def round_end(event_var):
    global roundended
    roundended = 1
    for userid in es.getUseridList():
        gamethread.cancelDelayed('checkFreeze_%s'%userid)
    PlayerCoords.clear()

    
def player_say(ev):

    if ev["text"].lower() == "!crosshair":
        uid = int(ev["userid"])
        if uid not in playerchoices:
            playerchoices[uid] = crosshairlist.values()[1]
            
        emenu = popuplib.easymenu("crosshairselect",None,selectPopupOption)
        emenu.settitle("Select your Crosshair")
        for x in crosshairlist:
            emenu.addoption((crosshairlist[x],x),x)
        emenu.send(uid)
        
def selectPopupOption(userid,choice,popup):
    playerchoices[userid] = choice[0]
    es.tell(userid,"You are now using the crosshair: %s" % choice[1])
    


def unload():
    ezrestrict.unrestrict('@all','#all')
    # Clean up our mess
    unfreeze.cancel()
    for player in playerlib.getPlayerList():
        gamethread.cancelDelayed('checkFreeze_%s'%player.userid)
        player.freeze = 0
        player.god = 0
        player.color = (255, 255, 255)
        
def player_death(event_var):
    uid = int(event_var["userid"])
    # add a check if they selfkilled
    if not roundended:
        es.server.queuecmd("est_spawn %s" % uid)
        # nt.spawnPlayer(uid)

def loop():
    for x in es.getUseridList():
        if x not in playerchoices:
            playerchoices[x] = crosshairlist.values()[1]
        if playerchoices[x] == 0:
            continue
        if playerlib.getPlayer(x).weapon == "weapon_scout":
            es.cexec(x,"r_screenoverlay %s" % playerchoices[x])
    gamethread.delayedname(2,"crosshairloop",loop)